////////////////////////////////////////////////////////////////////////////////
//	BaseViewer.cpp:	implementation of the CBaseViewer class.
//
//	This is the base class for all file viewing classes.  It contains definitions
//	of several basic data types used to extract binary data from the input files,
//	along with the basic routines that use these data types and some basic file
//	open/read/close processing common to any file viewer.
//
//	NOTE: This is assumed to be an open-source project, so individual method
//	notes appear before each method in the CPP file rather than here in the
//	.h file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileViewer.h"
#include "BaseViewer.h"

#define	EQUAL 0
#define IS_FILE_HEADER TRUE
#define IS_CHUNK_HEADER FALSE

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CBaseViewer::CBaseViewer()
{
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CBaseViewer::~CBaseViewer()
{
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Provides the main processing function of viewing a file.  There
//			  will be a subclass for each type of file viewed,
//
//   COMMENTS:
//
//	This class will perform the base functions of opening and closing the file
//	(etc.), and the file-type specific base classes will provide the actual
//	formatted views.
//
//	This function will not return until the entire file has been processed.
//	
//	  szFileName		The name of the input file to view
//	
//    RETURN VALUES:
//        None.  This class and its subclasses must handle all errors.
//
void CBaseViewer::ViewFile(char* szFileName)
{

}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Opens the file we are to view
//
//   COMMENTS:
//
//	This function opens the file passed in the constructor
//	
//    RETURN VALUES:
//		TRUE:	The file opened successfully
//		FALSE:	Otherwise
//
BOOL CBaseViewer::OpenTheFiles()
{
//
//			Open the input file
//
	if ((m_hInputFile = CreateFile((LPCTSTR) m_szInputFileName,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		(HANDLE)NULL)) == (HANDLE)-1)
	{
	    MessageBox( NULL, "Input file open failed.", NULL, MB_OK );
	    return FALSE;
	}
//
//			Get the size of the file and report empty files.
//
	m_dwFileSize = GetFileSize(m_hInputFile, NULL);
	if (m_dwFileSize == 0xFFFFFFFF)
	{
	    MessageBox( NULL, "GetFileSize failed!", NULL, MB_OK);
	    return FALSE;
	}
//			Create the output file name
//
	char	szDrive		[_MAX_DRIVE];
	char	szDir		[_MAX_DIR];
	char	szFname		[_MAX_FNAME];
	char	szExt		[_MAX_EXT];
	_splitpath (m_szInputFileName, szDrive, szDir, szFname, szExt);
	strcpy (m_szOutputFileName, "View_");
	strcat (m_szOutputFileName, szFname);
	strcat (m_szOutputFileName, ".txt");
//
//			Open the output file
//
	if ((m_hOutputFile = CreateFile((LPCTSTR) m_szOutputFileName,
		GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		CREATE_ALWAYS,
		0,
		(HANDLE)NULL)) == (HANDLE)-1)
	{
	    MessageBox( NULL, "Output file open failed.", NULL, MB_OK );
	    return FALSE;
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Close the file we have already processed
//
//   COMMENTS:
//
//	This function closes the file we opened in OpenTheFile above
//	
//    RETURN VALUES:
//		None
//
void CBaseViewer::CloseTheFiles()
{
	CloseHandle(m_hInputFile);
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read characters from the input file
//
//   COMMENTS:
//
//	This function reads the specified number of characters into the specified
//	input buffer from the current input file.
//	
//	  lpBuffer			Pointer to the location to put the read input
//	  dwLength			The number of bytes of data to read
//	
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadInput(void *lpBuffer,
							DWORD dwLength)
{
	DWORD	dwBytesRead;

	if (!ReadFile (m_hInputFile,(LPVOID)lpBuffer, dwLength, &dwBytesRead, NULL))
	{
	    MessageBox( NULL, "Read from the input file failed.", NULL, MB_OK );
		return FALSE;
	}

	if (dwBytesRead < dwLength)
	{
	    MessageBox( NULL, "End of file found unexpectedly.", NULL, MB_OK );
    	return FALSE;
	}

	return TRUE;
}



////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a UINT value from the input file
//
//   COMMENTS:
//
//	This function reads a UINT value into the specified passed parameter
//	from the current input file.
//	
//	  lpUINT			Pointer of the location of the UINT value to read
//	
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadWORD(WORD *lpUINT)
{
	return ReadInput (lpUINT, sizeof(WORD));
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a DWORD value from the input file
//
//   COMMENTS:
//
//	  lpDWORD			Pointer of the location of the DWORD value to read
//
//	This function reads a DWORD value into the specified passed parameter
//	from the current input file.
//	
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadDWORD (DWORD *lpDWORD)
{
	return ReadInput (lpDWORD, sizeof(DWORD));
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a Point2f value from the input file
//
//   COMMENTS:
//
//	This function reads a Point2f value into the specified passed parameter
//	from the current input file.
//	
//	  ThePoint			Pointer of the location of the Point2f value to read
//
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadPoint2f(struct Point2f *ThePoint)
{
	return ReadInput (ThePoint, sizeof(Point2f));
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a Vector3f value from the input file
//
//   COMMENTS:
//
//	  TheVector			Pointer of the location of the Vector3f value to read
//
//	This function reads a Vector3f value into the specified passed parameter
//	from the current input file.
//	
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadVector3f(struct Vector3f *TheVector)
{
	return ReadInput (TheVector, sizeof(Vector3f));
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a Matrix3f value from the input file
//
//   COMMENTS:
//
//	This function reads a Matrix3f value into the specified passed parameter
//	from the current input file.
//	
//	  TheMatrix			Pointer of the location of the Matrix3f value to read
//
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadMatrix3f(Matrix3f *TheMatrix)
{
	return ReadInput (TheMatrix, sizeof(Matrix3f));
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a Quaternion value from the input file
//
//   COMMENTS:
//
//	This function reads a Quaternion value into the specified passed parameter
//	from the current input file.
//	
//	  TheQuaternion		Pointer of the location of the Quaternion value to read
//
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadQuaternion(struct Quaternion *TheQuaternion)
{
	return ReadInput (TheQuaternion, sizeof(Quaternion));
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Read a float value from the input file
//
//   COMMENTS:
//
//	This function reads a float value into the specified passed parameter
//	from the current input file.
//	
//	  lpFloat			Pointer of the location of the float value to read
//
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::ReadFloat(float *lpFloat)
{
	return ReadInput (lpFloat, sizeof(float));
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write characters to the output file
//
//   COMMENTS:
//
//	This function writes the specified number of characters from the specified
//	input buffer to the current output file.
//	
//	  lpBuffer			Pointer of the location data to write tot he output file
//	  dwLength			The number of characters of data to write
//
//		RETURN VALUES:
//		TRUE:	If the file read is successful
//		FALSE:	Otherwise
//
BOOL CBaseViewer::WriteOutput (char *lpBuffer,
							   DWORD dwLength)
{
	DWORD	dwBytesWritten;

	if (! WriteFile (m_hOutputFile,(LPVOID)lpBuffer, dwLength, &dwBytesWritten, NULL))
	{
	    MessageBox( NULL, "Write to the output file failed.", NULL, MB_OK );
		return FALSE;
	}
	if (dwBytesWritten < dwLength)
	{
	    MessageBox( NULL, "Could not write all output to the output file.", NULL, MB_OK );
		return FALSE;
	}

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Return the name of the generated output file
//
//   COMMENTS:
//
//	This function fills the passed character string with the name fo the output
//	file created for this file veiwing.  Note: it is up to the calling routine
//	to insure that the passed szFileName is long enough to hold the name.
//	
//	  szFileName		Pointer of the location to put the output filename
//
//		RETURN VALUES:
//		None
//
void CBaseViewer::GetOutputFileName(char *szFileName)
{
	strcpy (szFileName, m_szOutputFileName);
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of an *.IM, *.PM, or *.KIN file chunk header.
//
//   COMMENTS:
//
//	This function dumps the contents of a header in a standard format.
//	Headers are made up of the following components
//
//		Chunk ID		4 characters
//		Chunk Size		Udword
//		File Type		4 characters	(only used by the file header)
//		Chunk Version	Udword			(not use by the file header)
//
//	  bIsFileHeader		File header (as opposed to chunk header) flag
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CBaseViewer::ViewChunkHeader(bool bIsFileHeader)
{
//
//			Read the chunk type and handle special conditions for the
//			initial file definition header.
//
	DWORD	dwChunkSize = 0;
	DWORD	dwFilePointer = m_dwFileSizeRead;
	char	szChunkID	[5];

	if (! ReadInput (szChunkID, 4))
		return FALSE;
	szChunkID[4] = '\0';

	if (! ReadDWORD (&dwChunkSize))
		return FALSE;

	if (bIsFileHeader)
	{
		if (! ReadInput (m_szFileType, 4))
			return FALSE;
		m_szFileType[4] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u] File  ID: %s,  Size: %08u (full size: %08u),  Type: %s\r\n",
				 dwFilePointer, szChunkID, dwChunkSize, dwChunkSize + 8, m_szFileType);
		m_dwTotalFileSize = dwChunkSize;
		m_dwFileSizeRead += strlen(szChunkID) + sizeof(DWORD) + strlen(m_szFileType);
	}
	else
	{
		if (! ReadDWORD (&m_dwVersion))
			return FALSE;
		if ((strcmp (szChunkID, "MATL") == EQUAL) ||
			(strcmp (szChunkID, "GEOM") == EQUAL))
			sprintf (m_OutputBuffer,
					 "[%08u]   Chunk ID: %s,  Size: %05u (next chunk: [%08u]),  Version: %u\r\n",
					 dwFilePointer, szChunkID, dwChunkSize, 
					 dwChunkSize + dwFilePointer + strlen(szChunkID) + sizeof(DWORD), m_dwVersion);
		else
			sprintf (m_OutputBuffer,
					 "[%08u] Chunk ID: %s,  Size: %05u (next chunk: [%08u]),  Version: %u\r\n",
					 dwFilePointer, szChunkID, dwChunkSize, 
					 dwChunkSize + dwFilePointer + strlen(szChunkID) + sizeof(DWORD), m_dwVersion);
		m_dwChunkSize = dwChunkSize;
		m_dwFileSizeRead += strlen(szChunkID) + sizeof(DWORD) + sizeof(DWORD);
	}
//
//			Write the found header to the output file
//
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;

	strcpy (m_szLastChunkRead, szChunkID);
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
