////////////////////////////////////////////////////////////////////////////////
//	IMViewer.cpp:	The class that implements the formatted output of an *.IM file
//
//	This class creates a formatted display of the contents of the input *IM or
//	*.PM file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileViewer.h"
#include "IMViewer.h"
#include <stdlib.h>

#define	EQUAL 0
#define IS_FILE_HEADER TRUE
#define IS_CHUNK_HEADER FALSE

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CIMViewer::CIMViewer()
{

}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CIMViewer::~CIMViewer()
{

}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Provides the main processing function of viewing an IM/PM file.
//
//   COMMENTS:
//
//	This function provides the main function of viewing an IM/PM file.
//	This class will provide the actual formatted views for IM files.
//
//	This function will not return until the entire file has been processed.
//	
//	  szFileName		The name of the file to view
//
//    RETURN VALUES:
//        None.  This class must handle all errors.
//
void CIMViewer::ViewFile(char* szFileName)
{
	m_dwTotalFileSize	= 0;
	m_dwFileSizeRead	= 0;
	m_dwTotalChunkCount	= 0;
	m_dwChunksFound		= 0;
//
//			Open the input and output files
//
	strcpy (m_szInputFileName, szFileName);
	if (! OpenTheFiles ())
	{
	   	MessageBox (NULL, "File open failed, returning from IMViewer", NULL, MB_OK );
		return;
	}
//
//			Report on the file we are about to view.
//
	sprintf (m_OutputBuffer, "           File name: %s\r\n\r\n",m_szInputFileName);
	WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer));
//
//			View the file and INFO headers, and check for expected values that
//			identify this as a Jet Indexed Mesh file.
//
	if (! ViewChunkHeader (IS_FILE_HEADER))
	{
		CloseTheFiles ();
		return;
	}
	if (strcmp (m_szLastChunkRead, "JIRF") != EQUAL)
	{
		strcpy (m_szMessage, "Cannot find \"JIRF\" Jet file ID, processing stopped.");
		WriteOutput (m_szMessage, strlen(m_szMessage));
	   	MessageBox (NULL, m_szMessage, NULL, MB_OK );
		CloseTheFiles ();
		return;
	}
	if (strcmp (m_szFileType, "IDXM") != EQUAL)
	{
		strcpy (m_szMessage, "Cannot find \"IDXM\" Jet Indexed/Progressive Mesh file type ID, processing stopped.");
		WriteOutput (m_szMessage, strlen(m_szMessage));
	   	MessageBox (NULL, m_szMessage, NULL, MB_OK );
		CloseTheFiles ();
		return;
	}
	
	if (! ViewChunkHeader (IS_CHUNK_HEADER))
	{
		CloseTheFiles ();
		return;
	}
//
//			View the contents of the INFO chunk
//
	if (strcmp (m_szLastChunkRead, "INFO") == EQUAL)
		ViewInfoChunk ();
	else
	{
		strcpy (m_szMessage, "Cannot find INFO chunk");
		WriteOutput (m_szMessage, strlen(m_szMessage));
	   	MessageBox (NULL, m_szMessage, NULL, MB_OK );
		CloseTheFiles ();
		return;
	}
//
//			View all the CHNK chunks (and their sub-chunks)
//
	while (m_dwFileSizeRead < m_dwTotalFileSize)
	{
		if (! ViewChunkHeader (IS_CHUNK_HEADER))
		{
			CloseTheFiles ();
			return;
		}
		if (strcmp (m_szLastChunkRead, "CHNK") == EQUAL)		// CHNK
		{
			if (! ViewAttributeChunk ())
			{
				CloseTheFiles ();
				return;
			}
		}
		else if (strcmp (m_szLastChunkRead, "INFL") == EQUAL)	// INFL
		{
			if (! ViewInfluenceChunk ())
			{
				CloseTheFiles ();
				return;
			}
		}
		else if (strcmp (m_szLastChunkRead, "ATCH") == EQUAL)	// ATCH
		{
			if (! ViewAttachmentChunk ())
			{
				CloseTheFiles ();
				return;
			}
		}
		else if (strcmp (m_szLastChunkRead, "NINF") == EQUAL)	// NINF
		{
			if (! ViewNeighborInfoChunk ())
			{
				CloseTheFiles ();
				return;
			}
		}
		else if (strcmp (m_szLastChunkRead, "PMIN") == EQUAL)	// PMIN
		{
			if (! ViewProgressiveInfoChunk ())
			{
				CloseTheFiles ();
				return;
			}
		}
		else
		{
			sprintf (m_szMessage, 
					 "Do not understand chunk type of [%s] at location [%u], processing terminated.",
					 m_szLastChunkRead, m_dwFileSizeRead);
		   	MessageBox (NULL, m_szMessage, NULL, MB_OK );
			CloseTheFiles ();
			return;
		}
	}
//
//			Report on what we have done and close the files.
//
	sprintf (m_OutputBuffer,
			 "\r\n           Total file size: %u,  Total chunks found: %u\r\n",
			 m_dwFileSizeRead, m_dwTotalChunkCount);
	WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer));
	CloseTheFiles ();

	return;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the INFO chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the INFO chunk in a standard format.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
BOOL CIMViewer::ViewInfoChunk()
{
	Vector3f	vPosition = {0.0, 0.0, 0.0};
	Quaternion	qRotation = {0.0, 0.0, 0.0, 0.0};
	DWORD		dwChunkCount  = 0;
	DWORD		dwFilePointer = m_dwFileSizeRead;
	DWORD		dwInflPerVertex = 0;
	DWORD		dwInflPerChunk = 0;
//
//			Read the position, rotatin, and chunk count

	if (! ReadVector3f (&vPosition))
		return FALSE;

	if (! ReadQuaternion (&qRotation))
		return FALSE;

	if (! ReadDWORD (&dwChunkCount))
		return FALSE;

	if (m_dwVersion == 102)				// handle changes for version 102
	{
		if (! ReadDWORD (&dwInflPerVertex))
			return FALSE;
		if (! ReadDWORD (&dwInflPerChunk))
			return FALSE;
	}
	sprintf (m_OutputBuffer,
			 "[%08u]     Position: [% f, % f, % f]\r\n",
			 dwFilePointer, vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;

	dwFilePointer += sizeof (Vector3f);

	sprintf (m_OutputBuffer,
			 "[%08u]     Rotation: [% f, % f, % f, % f]\r\n",
			 dwFilePointer, qRotation.fValue[0], qRotation.fValue[1], qRotation.fValue[2], qRotation.fValue[3]);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;

	dwFilePointer += sizeof (Quaternion);

	sprintf (m_OutputBuffer, 
			 "[%08u]     Number of CHNK type Chunks: %u\r\n",
			 dwFilePointer, dwChunkCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;

	dwFilePointer += sizeof (DWORD);

	if (m_dwVersion == 102)				// handle changes for version 102
	{
		sprintf (m_OutputBuffer, 
				 "[%08u]     Influence per Vertex: %u, Influence per Chunk: %u\r\n",
				 dwFilePointer, dwInflPerVertex, dwInflPerChunk);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (2 * sizeof (DWORD));
	}
	m_dwFileSizeRead = dwFilePointer;

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the CHNK chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the CHNK chunk in a standard format.
//	Attribute chunks are made up of the following components
//
//		Chunk index			Udword	(optional)
//		Material chunk		"MATL"
//		Geometry chunk		"GEOM"
//		Neighbor Info chunk	"NINF	(optional)
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
BOOL CIMViewer::ViewAttributeChunk ()
{
	DWORD		dwIndex = 0;
	DWORD		dwFilePointer = m_dwFileSizeRead;
//
//			View index value
//
	if (! ReadDWORD (&dwIndex))
		return FALSE;

	sprintf (m_OutputBuffer, 
			 "[%08u]     Chunks Index: %u\r\n",
			 dwFilePointer, dwIndex);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;

	m_dwFileSizeRead += sizeof (DWORD);
//
//			View the MATL chunk
//
	if (! ViewChunkHeader (IS_CHUNK_HEADER))
			return FALSE;
	if (! ViewMaterialChunk ())
			return FALSE;
	if (! ViewChunkHeader (IS_CHUNK_HEADER))
			return FALSE;
	if (! ViewGeometryChunk ())
			return FALSE;

	m_dwTotalChunkCount++;

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the MATL chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the MATL chunk in a standard format.
//	Attribute chunks are made up of the following components
//
//		Material attributes
//		Textures (zero or more)
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
BOOL CIMViewer::ViewMaterialChunk()
{
	Vector3f	vVector			= {0.0, 0.0, 0.0};
	float		fShininess		= 0.0;
	float		fTextureAmount	= 0.0;
	float		fOpacity		= 0.0;
	DWORD		dwPropertyCount	= 0;
	DWORD		dwTwoSided		= 0;
	DWORD		dwNumTextures	= 0;
	DWORD		dwTextureType	= 0;
	DWORD		dwNameSize		= 0;
	DWORD		dwPropertySize	= 0;
	DWORD		dwFilePointer	= m_dwFileSizeRead;
	char *		szAttribute []	= {"Ambient  ", "Difuse   ", "Specular ", "Emissive "};
	char		szTextureName	[MAX_PATH];
	char		szMaterialName	[MAX_PATH];
	char		szPropertyName	[MAX_PATH];
	char		szTypeName [1024];

	if (m_dwVersion == 102)
	{
//
//			View the material name size
//		
		if (! ReadDWORD (&dwNameSize))
			return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]         Material Name Size: %u\r\n",
				 dwFilePointer, dwNameSize);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (DWORD);
//
//			View the texture name
//		
		if (! ReadInput (szMaterialName, dwNameSize))
			return FALSE;
		szMaterialName[dwNameSize] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u]         Material Name:      %s\r\n",
				 dwFilePointer, szMaterialName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += dwNameSize;

		if (! ReadDWORD (&dwPropertyCount))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]     Property Count: %u\r\n",
				 dwFilePointer, dwPropertyCount);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (DWORD);

		for (DWORD nPropIndex = 0; nPropIndex < dwPropertyCount; nPropIndex++)
		{
//
//			View each property found
//		
			if (! ReadDWORD (&dwPropertySize))
				return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]         Property[%d] Name Size: %u\r\n",
					 dwFilePointer, nPropIndex, dwPropertySize);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (DWORD);

			if (! ReadInput (szPropertyName, dwPropertySize))
				return FALSE;
			szPropertyName[dwPropertySize] = '\0';
			sprintf (m_OutputBuffer,
					 "[%08u]         Property[%d] Name:      %s\r\n",
					 dwFilePointer, szPropertyName);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += dwPropertySize;
//
//			and the property's value
//		
			if (! ReadDWORD (&dwPropertySize))
				return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]         Property[%d] Name Size: %u\r\n",
					 dwFilePointer, nPropIndex, dwPropertySize);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (DWORD);

			if (! ReadInput (szPropertyName, dwPropertySize))
				return FALSE;
			szPropertyName[dwPropertySize] = '\0';
			sprintf (m_OutputBuffer,
					 "[%08u]         Property[%d] Name:      %s\r\n",
					 dwFilePointer, szPropertyName);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += dwPropertySize;
		}
	}
//
//			View two-sided value
//
	if (! ReadDWORD (&dwTwoSided))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     Two Sided:      %u\r\n",
			 dwFilePointer, dwTwoSided);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);


	if (m_dwVersion == 102)
	{
//
//			View another version 102 thingy
//		
		if (! ReadFloat (&fOpacity))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]     Opacity  :      %f\r\n",
				 dwFilePointer, fOpacity);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (float);
	}

//
//			View the next 4 RGB values
//
	for (int nIndex = 0; nIndex < 4; nIndex++)
	{
		if (! ReadVector3f (&vVector))
			return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]     %s:      [R:% f, G:% f, B:% f]\r\n",
				 dwFilePointer, szAttribute[nIndex], vVector.fValue[0], vVector.fValue[1], vVector.fValue[2]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;

		dwFilePointer += sizeof (Vector3f);
	}
//
//			View the shininess value
//
	if (! ReadFloat (&fShininess))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     Shininess:     % f\r\n",
			 dwFilePointer, fShininess);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (float);
//
//			View the number of textures value
//
	if (! ReadDWORD (&dwNumTextures))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     Number of textures: %u\r\n",
			 dwFilePointer, dwNumTextures);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);

	for (DWORD nTexture = 0; nTexture < dwNumTextures; nTexture++)
	{
//
//			View the texture type
//		
		if (! ReadDWORD (&dwTextureType))
			return FALSE;
//
//			Decode the texture type value
//
		switch (dwTextureType)
		{
		case -1:
			strcpy (szTypeName, "TEX_INVALID");			break;
		case 0:
			strcpy (szTypeName, "TEX_AMBIENT");			break;
		case 1:
			strcpy (szTypeName, "TEX_DIFFUSE");			break;
		case 2:
			strcpy (szTypeName, "TEX_SPECULAR");		break;
		case 3:
			strcpy (szTypeName, "TEX_SHINE");			break;
		case 4:
			strcpy (szTypeName, "TEX_SHINESTRENGTH");	break;
		case 5:
			strcpy (szTypeName, "TEX_SELFILLUM");		break;
		case 6:
			strcpy (szTypeName, "TEX_OPACITY");			break;
		case 7:
			strcpy (szTypeName, "TEX_FILTERCOLOR");		break;
		case 8:
			strcpy (szTypeName, "TEX_BUMP");			break;
		case 9:
			strcpy (szTypeName, "TEX_REFLECT");			break;
		case 10:
			strcpy (szTypeName, "TEX_REFRACT");			break;
		default:
			strcpy (szTypeName, "?? unknown ??");		break;
		}

		sprintf (m_OutputBuffer,
				 "[%08u]     Texture Type: %u (%s)\r\n",
				 dwFilePointer, dwTextureType, szTypeName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (DWORD);
//
//			View the texture name size
//		
		if (! ReadDWORD (&dwNameSize))
			return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]         Texture Name Size: %u\r\n",
				 dwFilePointer, dwNameSize);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (DWORD);
//
//			View the texture name
//		
		if (! ReadInput (szTextureName, dwNameSize))
			return FALSE;
		szTextureName[dwNameSize] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u]         Texture Name:      %s\r\n",
				 dwFilePointer, szTextureName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += dwNameSize;
//
//			View the texture Amount
//		
		if (! ReadFloat (&fTextureAmount))
			return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]         Texture Amount:   % f\r\n",
				 dwFilePointer, fTextureAmount);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (float);
	}

	m_dwFileSizeRead = dwFilePointer;
	
	return TRUE;

}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the GEOM chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the GEOM chunk in a standard format.
//	Attribute chunks are made up of the following components
//
//		Vertex location, normals, and text coordinate informaiton
//		Triangles vertex and normals information
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
BOOL CIMViewer::ViewGeometryChunk()
{
	Point2f		pTexCoord = {0.0, 0.0};
	Vector3f	vPosition = {0.0, 0.0, 0.0};

	float	fArea			= 0.0;
	DWORD	dwFlags			= 0;
	DWORD	dwIndexCount	= 0;
	DWORD	dwFaceNormCount	= 0;
	DWORD	dwFilePointer	= m_dwFileSizeRead;
	DWORD	dwVertexCount	= 0;
//	DWORD	dwTriangleCount	= 0;
	DWORD	dwMaxInfluences	= 0;
	DWORD	nVIndex;
	DWORD	nTIndex;
	WORD	dwVert[3]		= {0, 0, 0};
	char	szFlagName [1024];
//
//			new items for version 102
//
	m_dwTriangleCount = 0;
	if (m_dwVersion > 100)
	{
		if (! ReadDWORD (&dwFlags))
			return FALSE;
		if (! ReadFloat (&fArea))
			return FALSE;
		if (! ReadDWORD (&dwVertexCount))
			return FALSE;
		if (! ReadDWORD (&m_dwTriangleCount))
			return FALSE;
		if (! ReadDWORD (&dwIndexCount))
			return FALSE;
		if (! ReadDWORD (&dwFaceNormCount))
			return FALSE;
		if (m_dwVersion > 101)
		{
			if (! ReadDWORD (&dwMaxInfluences))
				return FALSE;
		}
//
//			Decode the flags value
//
		switch (dwFlags)
		{
		case 1:
			strcpy (szFlagName, "GC_POINTS");		break;
		case 2:
			strcpy (szFlagName, "GC_LINES");		break;
		case 3:
			strcpy (szFlagName, "GC_LINE_STRIP");	break;
		case 4:
			strcpy (szFlagName, "GC_TRIANGLES");	break;
		case 5:
			strcpy (szFlagName, "GC_TRI_STRIP");	break;
		case 6:
			strcpy (szFlagName, "GC_TRI_FAN");		break;
		default:
			strcpy (szFlagName, "?? unknown ??");	break;
		}

		sprintf (m_OutputBuffer,
				 "[%08u]     Flags: %u (%s)  Area: %f Vertex Count: %u Triangle Count: %u\r\n",
				 dwFilePointer, dwFlags, szFlagName, fArea, dwVertexCount, m_dwTriangleCount);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (sizeof (float) + (3 * (sizeof (DWORD))));
		if (m_dwVersion == 101)
		{
			sprintf (m_OutputBuffer,
					 "[%08u]     Index Count: %u  Face Normal Count: %u\r\n",
					 dwFilePointer, dwIndexCount, dwFaceNormCount);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += (2 * (sizeof (DWORD)));
		}
		else
		{
			sprintf (m_OutputBuffer,
					 "[%08u]     Index Count: %u  Face Normal Count: %u  Max Influences: %u\r\n",
					 dwFilePointer, dwIndexCount, dwFaceNormCount, dwMaxInfluences);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += (3 * (sizeof (DWORD)));
		}
	}
	else
	{
//
//			View the total area, number of verticies, and number of triangles
//		
		if (! ReadFloat (&fArea))
			return FALSE;

		if (! ReadDWORD (&dwVertexCount))
			return FALSE;

		if (! ReadDWORD (&m_dwTriangleCount))
			return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]     Area: % f   Vertex count: %u   Triangle count: %u\r\n",
				 dwFilePointer, fArea, dwVertexCount, m_dwTriangleCount);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (sizeof (float) + (2 * (sizeof (DWORD))));
	}
//
//			Display the Vertex data if there is any
//
	if (dwVertexCount > 0)
	{
		strcpy (m_szMessage, "               ---- VERTEX POSITIONS ----\r\n");
		if (! WriteOutput (m_szMessage, strlen(m_szMessage)))
			return FALSE;

		for (nVIndex = 0; nVIndex < dwVertexCount; nVIndex++)
		{
			if (! ReadVector3f (&vPosition))
				return FALSE;
			if (! ReadPoint2f (&pTexCoord))
				return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]     Vertex[%05u]: [% f, % f, % f]   Texture Coord: [% f, % f]\r\n",
					 dwFilePointer, nVIndex,
					 vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2],
					 pTexCoord.fValue[0], pTexCoord.fValue[1]);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += (5 * (sizeof (float)));
		}
	}
//
//			Display the Triangle data if there is any
//
	if (m_dwTriangleCount > 0)
	{
		strcpy (m_szMessage, "               ---- TRANGLE INDICES ----\r\n");
		if (! WriteOutput (m_szMessage, strlen(m_szMessage)))
			return FALSE;
		for (nTIndex = 0; nTIndex < m_dwTriangleCount; nTIndex++)
		{
			if (! ReadWORD (&dwVert[0]))
				return FALSE;
			if (! ReadWORD (&dwVert[1]))
				return FALSE;
			if (! ReadWORD (&dwVert[2]))
			return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]     Triangle[%05u]: [%05u, %05u, %05u]\r\n",
					 dwFilePointer, nTIndex, dwVert[0], dwVert[1], dwVert[2]);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += (3 * (sizeof (WORD)));
		}
	}
//
//			Display the Vertex normals if there are any
//
	if (dwVertexCount > 0)
	{
		strcpy (m_szMessage, "               ---- VERTEX NORMALS ----\r\n");
		if (! WriteOutput (m_szMessage, strlen(m_szMessage)))
			return FALSE;

		for (nVIndex = 0; nVIndex < dwVertexCount; nVIndex++)
		{
			if (! ReadVector3f (&vPosition))
				return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]     Vertex[%05u]: [% f, % f, % f]\r\n",
					 dwFilePointer, nVIndex,
					 vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += (3 * (sizeof (float)));
		}
	}
//
//			Display the Triangle normals if there are any
//
	if (m_dwTriangleCount > 0)
	{
		strcpy (m_szMessage, "               ---- TRIANGLE NORMALS ----\r\n");
		if (! WriteOutput (m_szMessage, strlen(m_szMessage)))
			return FALSE;

		for (nTIndex = 0; nTIndex < m_dwTriangleCount; nTIndex++)
		{
			if (! ReadVector3f (&vPosition))
				return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]     Triangle[%05u]: [% f, % f, % f]\r\n",
					 dwFilePointer, nTIndex,
					 vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += (3 * (sizeof (float)));
		}
	}
	m_dwFileSizeRead = dwFilePointer;
	
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the INFL chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the INFL chunk in a standard format.
//	Attribute chunks are made up of the following components
//
//		Bone definitions		(zero or more bones)
//			Bone definitions	Bone name, Bone parent name, Number of INFL chunks
//		Influence chunks		"INFL"
//			Chunk index
//			Vertex definitions
//				Index, weight, Position data
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
BOOL CIMViewer::ViewInfluenceChunk()
{
	Matrix3f	mOrientation		= {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
	Vector3f	vPosition			= {0.0, 0.0, 0.0};
	DWORD		dwFilePointer		= m_dwFileSizeRead;
	DWORD		dwBoneNameSize		= 0;
	DWORD		dwParentNameSize	= 0;
	DWORD		dwBoneCount			= 0;
	DWORD		dwInfluenceCount	= 0;
	DWORD		dwVertexCount		= 0;
	DWORD		dwChunkIndex		= 0;
	DWORD		dwVertexIndex		= 0;
	float		fWeight				= 0.0;
	char		szBoneName		[MAX_PATH];
	char		szParentName	[MAX_PATH];

//
//			View the number of bones
//		
	if (! ReadDWORD (&dwBoneCount))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     Bone Count: %u\r\n",
			 dwFilePointer, dwBoneCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);
//
//			View the the information on each bone
//		
	for (DWORD dwBoneIndex = 0; dwBoneIndex < dwBoneCount; dwBoneIndex++)
	{
//
//			View the bone name
//		
		if (! ReadDWORD (&dwBoneNameSize))
			return FALSE;

		if (! ReadInput (szBoneName, dwBoneNameSize))
			return FALSE;
		szBoneName[dwBoneNameSize] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u]     Bone Name size:   %2u, Name: %s\r\n",
				 dwFilePointer, dwBoneNameSize, szBoneName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (sizeof (DWORD) + dwBoneNameSize);
//
//			View the parent name
//		
		if (! ReadDWORD (&dwParentNameSize))
			return FALSE;

		if (! ReadInput (szParentName, dwParentNameSize))
			return FALSE;
		szParentName[dwParentNameSize] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u]     Parent Name size: %2u, Name: %s\r\n",
				 dwFilePointer, dwParentNameSize, szParentName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (sizeof (DWORD) + dwParentNameSize);
//
//			View the bone local position
//		
		if (! ReadVector3f (&vPosition))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]     Local Position:    [% f, % f, % f]\r\n",
				 dwFilePointer, vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (Vector3f);
//
//			View the bone local orientation
//		
		if (! ReadMatrix3f (&mOrientation))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]     Local Orientation: [% f, % f, % f]\r\n",
				 dwFilePointer,
				 mOrientation.fValue[0], mOrientation.fValue[1], mOrientation.fValue[2]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (Vector3f);
		sprintf (m_OutputBuffer,
				 "[%08u]                        [% f, % f, % f]\r\n",
				 dwFilePointer,
				 mOrientation.fValue[3], mOrientation.fValue[4], mOrientation.fValue[5]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (Vector3f);
		sprintf (m_OutputBuffer,
				 "[%08u]                        [% f, % f, % f]\r\n",
				 dwFilePointer,
				 mOrientation.fValue[6], mOrientation.fValue[7], mOrientation.fValue[8]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (Vector3f);
//
//			View the number of influence chunks for this bone
//		
		if (! ReadDWORD (&dwInfluenceCount))
			return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]     Influence Chunk Count: %u\r\n",
				 dwFilePointer, dwInfluenceCount);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (DWORD);
//
//			View the the information on each influence chunk for the current bone
//		
		for (DWORD dwInflIndex = 0; dwInflIndex < dwInfluenceCount; dwInflIndex++)
		{
//
//			View the influence index for this chunk
//		
			if (! ReadDWORD (&dwChunkIndex))
				return FALSE;

			sprintf (m_OutputBuffer,
					 "[%08u]         Chunk index:  %u\r\n",
					 dwFilePointer, dwChunkIndex);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (DWORD);
//
//			View the number of influence chunks for this bone
//		
			if (! ReadDWORD (&dwVertexCount))
				return FALSE;

			sprintf (m_OutputBuffer,
					 "[%08u]         Vertex Count: %u\r\n",
					 dwFilePointer, dwVertexCount);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (DWORD);
//
//			View the the information on each vertex for the current
//			influence chunk for the current bone
//		
			for (DWORD dwVIndex = 0; dwVIndex < dwVertexCount; dwVIndex++)
			{
//
//			View the vertex index
//		
			if (! ReadDWORD (&dwVertexIndex))
				return FALSE;

			sprintf (m_OutputBuffer,
					 "[%08u]             Vertex index: %u\r\n",
					 dwFilePointer, dwVertexIndex);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (DWORD);
//
//			View the vertex weight value
//
			if (! ReadFloat (&fWeight))
				return FALSE;

			sprintf (m_OutputBuffer,
					 "[%08u]                 Weight:        % f\r\n",
					 dwFilePointer, fWeight);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (float);
//
//			View the bone local position
//		
			if (! ReadVector3f (&vPosition))
				return FALSE;
			sprintf (m_OutputBuffer,
					 "[%08u]                 Local Position: [% f, % f, % f]\r\n",
					 dwFilePointer, vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return FALSE;
			dwFilePointer += sizeof (Vector3f);

			}	// end of this vertex
		}	// end of this influence chunk
	}	// end of this bone
	m_dwFileSizeRead = dwFilePointer;

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the ATCH chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the ATCH chunk in a standard format.
//	Attribute chunks are made up of the following components
//
//		Attachment Count		(zero or more attachments)
//			Attachment name		Char
//			Orientation			Matrix3f
//			Position			Vector3f
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
BOOL CIMViewer::ViewAttachmentChunk()
{
	Matrix3f	mOrientation		= {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
	Vector3f	vPosition			= {0.0, 0.0, 0.0};
	DWORD		dwFilePointer		= m_dwFileSizeRead;
	DWORD		dwAttachmentCount	= 0;
	DWORD		dwNameSize			= 0;
	char		szAttachmentName	[MAX_PATH];

	if (! ReadDWORD (&dwAttachmentCount))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     Attachment Count: %u\r\n",
			 dwFilePointer, dwAttachmentCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);

	for (DWORD dwAttachIndex = 0; dwAttachIndex < dwAttachmentCount; dwAttachIndex++)
	{
//
//			View the attachment name
//		
		if (! ReadDWORD (&dwNameSize))
			return FALSE;

		if (! ReadInput (szAttachmentName, dwNameSize))
			return FALSE;
		szAttachmentName[dwNameSize] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u]     Attachment Name size: %u, Name: %s\r\n",
				 dwFilePointer, dwNameSize, szAttachmentName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (sizeof (DWORD) + dwNameSize);
//
//			View the attachment orientation
//		
		if (! ReadMatrix3f (&mOrientation))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]     Orientation: [% f, % f, % f]\r\n",
				 dwFilePointer,
				 mOrientation.fValue[0], mOrientation.fValue[1], mOrientation.fValue[2]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]                  [% f, % f, % f]\r\n",
				 dwFilePointer,
				 mOrientation.fValue[3], mOrientation.fValue[4], mOrientation.fValue[5]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]                  [% f, % f, % f]\r\n",
				 dwFilePointer,
				 mOrientation.fValue[6], mOrientation.fValue[7], mOrientation.fValue[8]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (Matrix3f);
//
//			View the attachment position
//		
		if (! ReadVector3f (&vPosition))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]     Position:    [% f, % f, % f]\r\n",
				 dwFilePointer, vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;

		dwFilePointer += sizeof (Vector3f);
		m_dwFileSizeRead = dwFilePointer;
	}

	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the NINF chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the NINF chunk in a standard format.
//	Neighbor Informtion chunks are made up of indices if the three neighboring
//	triangles for each triangle.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CIMViewer::ViewNeighborInfoChunk ()
{
	DWORD	dwNeighbor0 = 0;
	DWORD	dwNeighbor1 = 0;
	DWORD	dwNeighbor2 = 0;
	DWORD	dwFilePointer = m_dwFileSizeRead;

	if (m_dwTriangleCount > 0)
	{
		strcpy (m_szMessage, "               ---- TRANGLE NEIGHBOURS ----\r\n");
		if (! WriteOutput (m_szMessage, strlen(m_szMessage)))	return false;
		for (DWORD nTIndex = 0; nTIndex < m_dwTriangleCount; nTIndex++)
		{
			if (! ReadDWORD (&dwNeighbor0))		return false;
			if (! ReadDWORD (&dwNeighbor1))		return false;
			if (! ReadDWORD (&dwNeighbor2))		return false;
			sprintf (m_OutputBuffer,
					 "[%08u]     Triangle[%05u]: [%08X, %08X, %08X]\r\n",
					 dwFilePointer, nTIndex, dwNeighbor0, dwNeighbor1, dwNeighbor2);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
				return false;
			dwFilePointer += (3 * (sizeof (DWORD)));
		}
		m_dwFileSizeRead = dwFilePointer;
	}
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the PMIN chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the PMIN chunk in a standard format.
//	Progressive Mesh Informtion chunks are made up of a number of collapses,
//	between differently scaled chunks giving a specific number of updates 
//	possible with the given chunks.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CIMViewer::ViewProgressiveInfoChunk ()
{
	DWORD	dwChangeCount	= 0;
	DWORD	dwCollapseCount	= 0;
	DWORD	dwChunkCount	= 0;
	DWORD	dwUpdateCount	= 0;
	DWORD	dwIndex			= 0;
	DWORD	dwFilePointer = m_dwFileSizeRead;
//
//			Read and display the counts that control the rest of the chunk
//
	if (! ReadDWORD (&dwChangeCount))		return false;
	if (! ReadDWORD (&dwCollapseCount))		return false;
	if (! ReadDWORD (&dwChunkCount))		return false;
	if (! ReadDWORD (&dwUpdateCount))		return false;

	sprintf (m_OutputBuffer, "[%08u]     Changes: %u\r\n",
			 dwFilePointer, dwChangeCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
	dwFilePointer += (sizeof (DWORD));

	sprintf (m_OutputBuffer, "[%08u]   Collapses: %u\r\n",
			 dwFilePointer, dwCollapseCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
	dwFilePointer += (sizeof (DWORD));

	sprintf (m_OutputBuffer, "[%08u]      Chunks: %u\r\n",
			 dwFilePointer, dwChunkCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
	dwFilePointer += (sizeof (DWORD));

	sprintf (m_OutputBuffer, "[%08u]     Updates: %u\r\n",
			 dwFilePointer, dwUpdateCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
	dwFilePointer += (sizeof (DWORD));
//
//			display the collapse information
//
	DWORD	dwVertexCount	= 0;
	DWORD	dwTriangleCount	= 0;
	DWORD	dwDifference	= 0;
	DWORD	dwChangeCount2	= 0;

	strcpy (m_szMessage, "               ---- COLLAPSES ----   Verticies  Triangles  Differences  Changes\r\n");
	if (! WriteOutput (m_szMessage, strlen(m_szMessage)))	return false;
	if (dwCollapseCount > 0)
	{
		for (dwIndex = 0; dwIndex < dwCollapseCount; dwIndex++)
		{
			if (! ReadDWORD (&dwVertexCount))		return false;
			if (! ReadDWORD (&dwTriangleCount))		return false;
			if (! ReadDWORD (&dwDifference))		return false;
			if (! ReadDWORD (&dwChangeCount2))		return false;

			sprintf (m_OutputBuffer, "[%08u]         Collapse [%04u]:  %04u       %04u       %04u         %04u\r\n",
					 dwFilePointer, dwIndex, dwVertexCount, dwTriangleCount, dwDifference, dwChangeCount2);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
			dwFilePointer += ((4 * sizeof (DWORD)));
		}
	}
//
//			Display the Chunk information
//
	DWORD	dwID			= 0;
	DWORD	dwCollapseCount2= 0;

	strcpy (m_szMessage, "               ---- CHUNKS ----      Chunk ID   Collapses\r\n");
	if (! WriteOutput (m_szMessage, strlen(m_szMessage)))	return false;
	if (dwChunkCount > 0)
	{
		for (dwIndex = 0; dwIndex < dwChunkCount; dwIndex++)
		{
			if (! ReadDWORD (&dwID))				return false;
			if (! ReadDWORD (&dwCollapseCount2))	return false;

			sprintf (m_OutputBuffer, "[%08u]            Chunk [%04u]:  %04u       %04u\r\n",
					 dwFilePointer, dwIndex, dwID, dwCollapseCount2);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
			dwFilePointer += ((2 * sizeof (DWORD)));
		}
	}
//
//			Display the update information
//
	DWORD	ChunkCount2	= 0;

	strcpy (m_szMessage, "               ---- UPDATES ----     Chunks\r\n");
	if (! WriteOutput (m_szMessage, strlen(m_szMessage)))	return false;
	if (dwChunkCount > 0)
	{
		for (dwIndex = 0; dwIndex < dwUpdateCount; dwIndex++)
		{
			if (! ReadDWORD (&ChunkCount2))		return false;

			sprintf (m_OutputBuffer, "[%08u]           Update [%04u]:  %04u\r\n",
					 dwFilePointer, dwIndex, ChunkCount2);
			if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))	return false;
			dwFilePointer += (sizeof (DWORD));
		}
	}
	m_dwFileSizeRead = dwFilePointer;
	
	return true;
}

////////////////////////////////////////////////////////////////////////////////
