////////////////////////////////////////////////////////////////////////////////
//	KINViewer.cpp:	The class that implements the formatted output of an *.KIN file
//
//	This class creates a formatted display of the contents of the input *KIN file
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileViewer.h"
#include "KINViewer.h"

#define	EQUAL 0
#define IS_FILE_HEADER TRUE
#define IS_CHUNK_HEADER FALSE

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CKINViewer::CKINViewer()
{
	m_dwTotalFileSize = 0;
	m_nBoneID = 0;
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CKINViewer::~CKINViewer()
{

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Provides the main processing function of viewing a KIN file.
//
//   COMMENTS:
//
//	This function provides the main function of viewing a KIN file.
//	This class will provide the actual formatted views for KIN files.
//
//	This function will not return until the entire file has been processed.
//	
//	  szFileName		The name of the file to view
//
//    RETURN VALUES:
//        None.  This class must handle all errors.
//
void CKINViewer::ViewFile(char *szFileName)
{
	char	szNextChunkType [5];

	m_dwTotalFileSize	= 0;
	m_dwFileSizeRead	= 0;
	m_dwTotalChunkCount	= 0;
	m_dwChunksFound		= 0;
//
//			Open the input and output files
//
	strcpy (m_szInputFileName, szFileName);
	if (! OpenTheFiles ())
	{
	   	MessageBox (NULL, "File open failed, returning from KINViewer", NULL, MB_OK );
		return;
	}
//
//			Report on the file we are about to view.
//
	sprintf (m_OutputBuffer, "           File name: %s\r\n\r\n",m_szInputFileName);
	WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer));
//
//			View the file and INFO headers, and check for expected values that
//			identify this as a Jet Indexed Mesh file.
//
	if (! ViewChunkHeader (IS_FILE_HEADER))
	{
		CloseTheFiles ();
		return;
	}
	if (strcmp (m_szLastChunkRead, "JIRF") != EQUAL)
	{
		strcpy (m_szMessage, "Cannot find \"JIRF\" Jet file ID, processing stopped.");
		WriteOutput (m_szMessage, strlen(m_szMessage));
	   	MessageBox (NULL, m_szMessage, NULL, MB_OK );
		CloseTheFiles ();
		return;
	}
	if (strcmp (m_szFileType, "ANIM") != EQUAL)
	{
		strcpy (m_szMessage, "Cannot find \"ANIM\" Jet animation file type ID, processing stopped.");
		WriteOutput (m_szMessage, strlen(m_szMessage));
	   	MessageBox (NULL, m_szMessage, NULL, MB_OK );
		CloseTheFiles ();
		return;
	}
//
//			View the high level chunk types
//
	strcpy (szNextChunkType, "INFO");
	while (m_dwFileSizeRead < m_dwTotalFileSize)
	{
		if (! ViewChunkHeader (IS_CHUNK_HEADER))	//	Read the next chunk header
		{
			CloseTheFiles ();
			return;
		}
		if ((strcmp (m_szLastChunkRead, "INFO") == EQUAL) &&
			(strcmp (m_szLastChunkRead, szNextChunkType) == EQUAL))
		{
			ViewInfoChunk ();						// Process the INFO chunk
			strcpy (szNextChunkType, "EVNT");
		}
		else if ((strcmp (m_szLastChunkRead, "EVNT") == EQUAL) &&
				 (strcmp (m_szLastChunkRead, szNextChunkType) == EQUAL))
		{
			ViewEventChunk ();						// Process the Event chunk
			strcpy (szNextChunkType, "SKEL");
		}
		else if ((strcmp (m_szLastChunkRead, "SKEL") == EQUAL) &&
				 (strcmp (m_szLastChunkRead, szNextChunkType) == EQUAL))
		{
			ViewSkelChunk ();						// Process the Skeleton type
			strcpy (szNextChunkType, "FRAM");
		}
		else if ((strcmp (m_szLastChunkRead, "FRAM") == EQUAL) &&
				 (strcmp (m_szLastChunkRead, szNextChunkType) == EQUAL))
		{
			ViewFrameChunk ();						// Process the Skeleton type
		}
		else
		{
			sprintf (m_szMessage, "Cannot find %s chunk.", szNextChunkType);
			WriteOutput (m_szMessage, strlen(m_szMessage));
	   		MessageBox (NULL, m_szMessage, NULL, MB_OK );
			CloseTheFiles ();
			return;
		}
	}
//
//			Report on what we have done and close the files.
//
	sprintf (m_OutputBuffer,
			 "\r\n           Total file size: %u,\r\n",
			 m_dwFileSizeRead);
	WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer));
	CloseTheFiles ();

	return;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the INFO chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the INFO chunk in a standard format.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CKINViewer::ViewInfoChunk()
{
	DWORD	dwFilePointer	= m_dwFileSizeRead;
	DWORD	dwNameSize		= 0;
	DWORD	dwFrameCount	= 0;
	DWORD	dwFrameRate		= 0;
	float	fMetricScale	= 0.0;

	char	szFileName	[MAX_PATH];
//
//			View the file name size
//		
	if (! ReadDWORD (&dwNameSize))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     File Name Size: %u\r\n",
			 dwFilePointer, dwNameSize);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);
//
//			View the file name
//		
	if (! ReadInput (szFileName, dwNameSize))
		return FALSE;
	szFileName[dwNameSize] = '\0';
	sprintf (m_OutputBuffer,
			 "[%08u]     Material Name:  %s\r\n",
			 dwFilePointer, szFileName);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += dwNameSize;
//
//			View the Frame Count
//
	if (! ReadDWORD (&dwFrameCount))	return FALSE;
	sprintf (m_OutputBuffer, 
			 "[%08u]     Number of Frames:   %u\r\n",
			 dwFilePointer, dwFrameCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	m_dwFileSizeRead += sizeof (DWORD);
//
//			View the Frame Rate
//
	if (! ReadDWORD (&dwFrameRate))	return FALSE;
	sprintf (m_OutputBuffer, 
			 "[%08u]     Frame Rate:         %u\r\n",
			 dwFilePointer, dwFrameRate);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	m_dwFileSizeRead += sizeof (DWORD);
//
//			View the Metric Sacle Value
//
	if (! ReadFloat (&fMetricScale))	return FALSE;
	sprintf (m_OutputBuffer,
			 "[%08u]     Metric Scale Value: %f\r\n",
			 dwFilePointer, fMetricScale);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (float);

	m_dwFileSizeRead = dwFilePointer;

	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the EVNT chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the EVNT chunk in a standard format.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CKINViewer::ViewEventChunk()
{
	DWORD	dwFilePointer	= m_dwFileSizeRead;
	DWORD	dwEventCount	= 0;
	DWORD	dwFrameNo		= 0;
	DWORD	dwType			= 0;
	DWORD	dwNameSize		= 0;
	char	szEventName [1024];
//
//			View the Event Count
//
	if (! ReadDWORD (&dwEventCount))	return FALSE;
	sprintf (m_OutputBuffer, 
			 "[%08u]     Number of Events:   %u\r\n",
			 dwFilePointer, dwEventCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);
//
//			View each event found
//		
	for (DWORD nEventIndex = 0; nEventIndex < dwEventCount; nEventIndex++)
	{
//
//			View the frame number and type
//		
		if (! ReadDWORD (&dwFrameNo))	return FALSE;
		if (! ReadDWORD (&dwType))		return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]         Event[%d] Frame: %u,  Type: %u\r\n",
				 dwFilePointer, nEventIndex, dwFrameNo, dwType);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += (2 * sizeof (DWORD));
//
//			View the event name size
//
		if (! ReadDWORD (&dwNameSize))	return FALSE;

		sprintf (m_OutputBuffer,
				 "[%08u]                   Event Name Size: %u\r\n",
				 dwFilePointer, dwNameSize);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (DWORD);
//
//			View the event name
//		
		if (! ReadInput (szEventName, dwNameSize))
			return FALSE;
		szEventName[dwNameSize] = '\0';
		sprintf (m_OutputBuffer,
				 "[%08u]                   Event Name:      %s\r\n",
				 dwFilePointer, szEventName);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += dwNameSize;
	}

	m_dwFileSizeRead = dwFilePointer;
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the SKEL chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the SKEL chunk in a standard format.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CKINViewer::ViewSkelChunk()
{
	if (! ViewChunkHeader (IS_CHUNK_HEADER))
		return false;
	strcpy (m_szBoneIndent, "| ");
	if (! ViewBoneChunk ())
		return false;
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the BONE chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the BONE chunk in a standard format.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CKINViewer::ViewBoneChunk()
{
	DWORD	dwFilePointer	= m_dwFileSizeRead;

	DWORD	dwNameSize		= 0;
	DWORD	dwChildCount	= 0;
	char	szBoneName		[1024];

//
//			View the bone name size
//		
	if (! ReadDWORD (&dwNameSize))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     %s[%05u] Bone Name Size: %u\r\n",
			 dwFilePointer, m_szBoneIndent, m_nBoneID, dwNameSize);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);
//
//			View the file name
//		
	if (! ReadInput (szBoneName, dwNameSize))
		return FALSE;
	szBoneName[dwNameSize] = '\0';
	sprintf (m_OutputBuffer,
			 "[%08u]     %s        Bone Name:      %s\r\n",
			 dwFilePointer, m_szBoneIndent, szBoneName);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += dwNameSize;
//
//			View the child count
//		
	if (! ReadDWORD (&dwChildCount))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     %s        child bones:    %u\r\n",
			 dwFilePointer, m_szBoneIndent, dwChildCount);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);

	m_nBoneID++;					// increment the ID for this bone.
	m_dwFileSizeRead = dwFilePointer;
//
//			View the child bones
//
	strcat (m_szBoneIndent, "| ");
	for (DWORD nBoneIndex = 0; nBoneIndex < dwChildCount; nBoneIndex++)
	{
		if (! ViewChunkHeader (IS_CHUNK_HEADER))
			return false;
		if (! ViewBoneChunk ())
			return false;
	}
	m_szBoneIndent[strlen(m_szBoneIndent) - 2] = '\0';
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: View the contents of the FRAM chuck.
//
//   COMMENTS:
//
//	This function dumps the contents of the FRAM chunk in a standard format.
//
//    RETURN VALUES:
//		TRUE:	If the file contains the required characters to view
//		FALSE:	If the file does not contain enough characters to view
//
bool CKINViewer::ViewFrameChunk()
{
	DWORD	dwFilePointer	= m_dwFileSizeRead;

	Vector3f	vPosition	= {0.0, 0.0, 0.0};
	Quaternion	qRotation	= {0.0, 0.0, 0.0, 0.0};
	DWORD		dwFrameNo	= 0;

//
//			View the frame number
//		
	if (! ReadDWORD (&dwFrameNo))
		return FALSE;

	sprintf (m_OutputBuffer,
			 "[%08u]     Frame number: %u\r\n",
			 dwFilePointer, dwFrameNo);
	if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
		return FALSE;
	dwFilePointer += sizeof (DWORD);
//
//			View the bone animations
//
	for (int nBoneIndex = 0; nBoneIndex < m_nBoneID; nBoneIndex++)
	{
		if (! ReadVector3f (&vPosition))		// handle the position
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]        [%05u] Bone Position: [% f, % f, % f]\r\n",
				 dwFilePointer, (nBoneIndex), vPosition.fValue[0], vPosition.fValue[1], vPosition.fValue[2]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;
		dwFilePointer += sizeof (Vector3f);

		if (! ReadQuaternion (&qRotation))
			return FALSE;
		sprintf (m_OutputBuffer,
				 "[%08u]                Rotation:      [% f, % f, % f, % f]\r\n",
				 dwFilePointer, qRotation.fValue[0], qRotation.fValue[1], qRotation.fValue[2], qRotation.fValue[3]);
		if (! WriteOutput (m_OutputBuffer, strlen(m_OutputBuffer)))
			return FALSE;

		dwFilePointer += sizeof (Quaternion);
	}



	m_dwFileSizeRead = dwFilePointer;
	return true;
}
